(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 9.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    112050,       3202]
NotebookOptionsPosition[    107874,       3085]
NotebookOutlinePosition[    108233,       3101]
CellTagsIndexPosition[    108190,       3098]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell["\<\
Distance-Matrix Cluster Analysis

Warning: needs at least two entries in the distance matrix

-

Distance-Matrix Basics and Examples

FindDistanceMatrix[]
Args:
Data vectors to find distances between
Distance function for those vectors (default: Euclidean distance)
Returns:
Distance matrix

Mathematica has a built-in function that also does this: DistanceMatrix[]

CheckDistanceMatrix[]
Args:
Object to be checked for being a distance matrix
Returns:
Whether or not a possible distance matrix

DMExample[]
Args:
Data source
What in the data source: \
\[OpenCurlyDoubleQuote]Names\[CloseCurlyDoubleQuote], \
\[OpenCurlyDoubleQuote]Vectors\[CloseCurlyDoubleQuote], \
\[OpenCurlyDoubleQuote]DistMat\[CloseCurlyDoubleQuote]

Data sources:
\[OpenCurlyDoubleQuote]WikiUP\[CloseCurlyDoubleQuote]: UPGMA - Wikipedia, the \
free encyclopedia - https://en.wikipedia.org/wiki/UPGMA
\[OpenCurlyDoubleQuote]WikiNJ\[CloseCurlyDoubleQuote]: Neighbor joining - \
Wikipedia, the free encyclopedia - \
https://en.wikipedia.org/wiki/Neighbor_joining
\[OpenCurlyDoubleQuote]WikiDM\[CloseCurlyDoubleQuote]: Distance matrix - \
Wikipedia, the free encyclopedia - \
https://en.wikipedia.org/wiki/Distance_matrix

-

Mathematica\[CloseCurlyQuote]s built-in distance functions. All have two \
args, both vectors of numbers.
For vectors x, y, with operations being component by component
EuclideanDistance = sqrt(sum of (x - y)^2)
SquaredEuclideanDistance = sum of (x - y)^2
ManhattanDistance = sum of |x - y|
ChessboardDistance = max of |x - y|
BrayCurtisDistance = (sum of |x - y|) / (sum of |x + y|)
CanberraDistance = sum of |x - y|/(|x| + |y|)

-

Distance-matrix fitting

DMFit[]
Args:
Distance matrix
Number of dimensions of the fitted data vectors
Distance function for those vectors (default: Euclidean distance)
Returns:
Fitted data vectors

Multidimensional Scaling

DMMDS[]
Args:
Distance matrix
Number of dimensions of the fitted data vectors
Returns:
Fitted data vectors (Euclidean distance)

DMSqMDS[]
Like above, but with the square of the Euclidean distance.

These algorithms involve turning Euclidean distances |xi - xj| into inner \
products xi.xj (more strictly xi and xj with average x subtracted out), There \
do not seem to be comparable algorithms for Mma\[CloseCurlyQuote]s other \
distance functions.

For all of these functions, the fitted data vectors have average = zero vector.

-

K-Medoid Clustering (PAM)
Does Partitioning Around Medoids using a distance matrix

Source: k-medoids - Wikipedia, the free encyclopedia - \
https://en.wikipedia.org/wiki/K-medoids

DMKMedoids[]
Args:
Distance matrix
Indices of initial medoids
Returns:
{Medoid indices, clusters as lists of indices of input values, indices of \
inputs\[CloseCurlyQuote] clusters}

RandomIndices[]
Args:
Number of indices: they range from 1 to n
How many distinct ones to choose (m)
Uses RandomReal[{0,1},m]
Returns:
Random selection of m distinct indices

-

Neighbor-joining algorithms

NeighborJoin[]
Args:
Distance matrix
Type: \[OpenCurlyDoubleQuote]WPGMA\[CloseCurlyDoubleQuote], \
\[OpenCurlyDoubleQuote]UPGMA\[CloseCurlyDoubleQuote], \
\[OpenCurlyDoubleQuote]NeiSaitou\[CloseCurlyDoubleQuote]
Returns:
Tree with branch nodes
{index, distance1, distance2, leafnode1, leafnode2}
and leaf nodes
(index)

NJLeafCount[]
Args:
Tree
Returns:
Number of leaf nodes

NJPathLengths[]
Args:
Tree
Returns:
{List of root-to-leaf distances: {#,dist}, list of leaf-to-leaf distances: \
{#1,#2,dist}}

NJMakeDistanceMatrix[]
Args:
Tree
Returns:
Distance matrix calculated from the tree

NJFlatten[]
Args:
Tree
Returns:
{# leaf nodes, branch nodes in order of their indices: {distance1, distance2, \
leafnode1 index, leafnode2 index}}
Branch nodes\[CloseCurlyQuote] indices start after the leaf nodes\
\[CloseCurlyQuote] ones

NJMakeMmaGraph[]
Args:
Tree
Returns:
Mathematica graph object

NJUnflatten[]
Args:
Flattened Tree, like from NJFlatten[]
Returns:
Tree

NJDumpTree[]
Args:
Tree
Names of leaf nodes (default: 1 to number of leaf nodes)
Returns:
Prettyprinted version of the tree with named leaf nodes

-

Fitch-Margoliash least-squares tree fitting

Tree format:
{links}
where each link is {node1, node2}

Unlike the NJ one, it does not include distances

NJtoFMTree[]
Args:
Tree from a neighbor-joining algorithm
Returns:
{FM tree, distance for each link}

The NJ tree\[CloseCurlyQuote]s root is edited out.

FMTreeExpand[]:
Args:
FM Tree
Label for expanded tree information (\[OpenCurlyDoubleQuote]trxp\
\[CloseCurlyDoubleQuote])
trxp[\[OpenCurlyDoubleQuote]NumLinks\[CloseCurlyDoubleQuote]] = how many \
links
trxp[\[OpenCurlyDoubleQuote]LinkNodes\[CloseCurlyDoubleQuote], link index] = \
the FM-tree input: list of pairs of nodes
trxp[\[OpenCurlyDoubleQuote]LinkOther\[CloseCurlyDoubleQuote], link index, \
node] = other node for that link
trxp[\[OpenCurlyDoubleQuote]NodeList\[CloseCurlyDoubleQuote]] = the tree\
\[CloseCurlyQuote]s nodes
trxp[\[OpenCurlyDoubleQuote]NodeLinks\[CloseCurlyDoubleQuote], node] = list \
of indices of that node\[CloseCurlyQuote]s links
trxp[\[OpenCurlyDoubleQuote]NodeOther\[CloseCurlyDoubleQuote], node, link \
indices] = list of other link indices for that node
Returns:
Whether the input tree was OK

FMMakeMmaGraph[]
Args:
FM Tree
Link labels (optional: default is the links index numbers)
Returns:
Mathematica graph object

FMCycles[]
Args:
FM Tree
Returns:
How many cycles were detected (ought to be zero for phylogeny applications)

FMPaths[]
Args:
FM Tree
Returns:
List of paths:
{leafnode1, link, node, link, node, ..., link, leafnode2}

FMPathMatrix[]
Args:
Paths from FMPaths[]
Returns:
Sparse array with dimensions
{leaf nodes, leaf nodes, links}

FMDMCalc[]
Args:
Tree
Distances for each link
Returns:
Calculated distance matrix

FMDMCheckRMS[]
Args:
Tree
Distances for each link
Distance matrix
Returns:
Root-mean-square difference

Fit to distance matrix

FMDMFit[]
Args:
Tree
Distance matrix
Returns:
{Distances for each link, RMS difference in the DM}

Building trees with the FM method

FMIncremental[]
Args:
Distance matrix
Returns:
{Tree, {distances, RMS difference}}

FMImproveByFlipping[]
Args:
Original tree
Distance matrix
Returns:
{Tree, {distances, RMS difference}}\
\>", "Text"],

Cell[CellGroupData[{

Cell["Distance-Matrix Basics and Examples", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"FindDistanceMatrix", "[", 
   RowBox[{"vectors_", ",", 
    RowBox[{"distfunc_:", "EuclideanDistance"}]}], "]"}], " ", ":=", " ", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"distfunc", "[", 
     RowBox[{"v1", ",", "v2"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"v1", ",", "vectors"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"v2", ",", "vectors"}], "}"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"CheckDistanceMatrix", "::", "notsquare"}], " ", "=", " ", 
   "\"\<The input is not a square matrix\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"CheckDistanceMatrix", "::", "notpossd"}], " ", "=", " ", 
   "\"\<The input is not positive semidefinite\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"CheckDistanceMatrix", "::", "notsym"}], " ", "=", " ", 
   "\"\<The input is not symmetric\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"CheckDistanceMatrix", "::", "notzerodiag"}], " ", "=", " ", 
   "\"\<The input's diagonal is not all zeros\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"CheckDistanceMatrix", "[", "distmat_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "dims", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"dims", " ", "=", " ", 
      RowBox[{"Dimensions", "[", "distmat", "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"Length", "[", "dims", "]"}], " ", "\[NotEqual]", " ", "2"}],
         " ", "||", " ", 
        RowBox[{
         RowBox[{"Length", "[", 
          RowBox[{"Union", "[", "dims", "]"}], "]"}], " ", "\[NotEqual]", " ",
          "1"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Message", "[", 
         RowBox[{"CheckDistanceMatrix", "::", "notsquare"}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Return", "[", "False", "]"}]}]}], "\[IndentingNewLine]", 
      "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"!", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"Min", "[", 
            RowBox[{"Flatten", "[", "distmat", "]"}], "]"}], 
           "\[GreaterEqual]", " ", "0"}], ",", "True", ",", " ", "False", ",",
           " ", "False"}], "]"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Message", "[", 
         RowBox[{"CheckDistanceMatrix", "::", "notpossd"}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Return", "[", "False", "]"}]}]}], "\[IndentingNewLine]", 
      "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"distmat", " ", "=!=", " ", 
        RowBox[{"Transpose", "[", "distmat", "]"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Message", "[", 
         RowBox[{"CheckDistanceMatrix", "::", "notsym"}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Return", "[", "False", "]"}]}]}], "\[IndentingNewLine]", 
      "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"!", 
        RowBox[{"(", 
         RowBox[{"And", " ", "@@", " ", 
          RowBox[{"(", " ", 
           RowBox[{
            RowBox[{
             RowBox[{"(", 
              RowBox[{"#", "\[Equal]", "0"}], ")"}], "&"}], " ", "/@", " ", 
            RowBox[{"Diagonal", "[", "distmat", "]"}]}], " ", ")"}]}], " ", 
         ")"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Message", "[", 
         RowBox[{"CheckDistanceMatrix", "::", "notzerodiag"}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Return", "[", "False", "]"}]}]}], "\[IndentingNewLine]", 
      "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Return", "[", "True", "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DMExample", "[", 
    RowBox[{"\"\<WikiUP\>\"", ",", "\"\<Names\>\""}], "]"}], " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{
    "\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\"", ",", "\"\<d\>\"", ",", 
     "\"\<e\>\""}], "}"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DMExample", "[", 
    RowBox[{"\"\<WikiUP\>\"", ",", "\"\<DistMat\>\""}], "]"}], " ", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "17", ",", "21", ",", "31", ",", "23"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"17", ",", "0", ",", "30", ",", "34", ",", "21"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"21", ",", "30", ",", "0", ",", "28", ",", "39"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"31", ",", "34", ",", "28", ",", "0", ",", "43"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"23", ",", "21", ",", "39", ",", "43", ",", "0"}], "}"}]}], 
    "}"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DMExample", "[", 
    RowBox[{"\"\<WikiNJ\>\"", ",", "\"\<Names\>\""}], "]"}], " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{
    "\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\"", ",", "\"\<d\>\"", ",", 
     "\"\<e\>\""}], "}"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DMExample", "[", 
    RowBox[{"\"\<WikiNJ\>\"", ",", "\"\<DistMat\>\""}], "]"}], " ", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "5", ",", "9", ",", "9", ",", "8"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"5", ",", "0", ",", "10", ",", "10", ",", "9"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"9", ",", "10", ",", "0", ",", "8", ",", "7"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"9", ",", "10", ",", "8", ",", "0", ",", "3"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"8", ",", "9", ",", "7", ",", "3", ",", "0"}], "}"}]}], "}"}]}],
   ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DMExample", "[", 
    RowBox[{"\"\<WikiDM\>\"", ",", "\"\<Names\>\""}], "]"}], " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{
    "\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\"", ",", "\"\<d\>\"", ",", 
     "\"\<e\>\"", ",", "\"\<f\>\""}], "}"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DMExample", "[", 
    RowBox[{"\"\<WikiDM\>\"", ",", "\"\<Vectors\>\""}], "]"}], " ", "=", " ", 
   
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"69", ",", "212"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "43"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"122", ",", "45"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"121", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"201", ",", "23"}], "}"}]}], "}"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DMExample", "[", 
    RowBox[{"\"\<WikiDM\>\"", ",", "\"\<DistMat\>\""}], "]"}], " ", "=", " ", 
   
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "0", ",", "184", ",", "222", ",", "177", ",", "216", ",", "231"}], 
      "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
      "184", ",", "0", ",", "45", ",", "123", ",", "128", ",", "200"}], "}"}],
      ",", 
     RowBox[{"{", 
      RowBox[{
      "222", ",", "45", ",", "0", ",", "129", ",", "121", ",", "203"}], "}"}],
      ",", 
     RowBox[{"{", 
      RowBox[{"177", ",", "123", ",", "129", ",", "0", ",", "46", ",", "83"}],
       "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"216", ",", "128", ",", "121", ",", "46", ",", "0", ",", "83"}],
       "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"231", ",", "200", ",", "203", ",", "83", ",", "83", ",", "0"}],
       "}"}]}], "}"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Mean", "[", 
   RowBox[{"Flatten", "[", 
    RowBox[{"Abs", "[", 
     RowBox[{"N", "[", 
      RowBox[{
       RowBox[{"DMExample", "[", 
        RowBox[{"\"\<WikiDM\>\"", ",", "\"\<DistMat\>\""}], "]"}], " ", "-", 
       " ", 
       RowBox[{"FindDistanceMatrix", "[", 
        RowBox[{
         RowBox[{"DMExample", "[", 
          RowBox[{"\"\<WikiDM\>\"", ",", "\"\<Vectors\>\""}], "]"}], ",", 
         "EuclideanDistance"}], "]"}]}], "]"}], "]"}], "]"}], "]"}], "/", 
  RowBox[{"(", 
   RowBox[{"1", "-", 
    RowBox[{"1", "/", 
     RowBox[{"Length", "[", 
      RowBox[{"DMExample", "[", 
       RowBox[{"\"\<WikiDM\>\"", ",", "\"\<Vectors\>\""}], "]"}], "]"}]}]}], 
   ")"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"ListPlot", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"Labeled", "@@", "#"}], "&"}], " ", "/@", " ", 
      RowBox[{"Thread", "[", 
       RowBox[{
        RowBox[{"DMExample", "[", 
         RowBox[{"\"\<WikiDM\>\"", ",", "\"\<Vectors\>\""}], "]"}], "\[Rule]", 
        RowBox[{"DMExample", "[", 
         RowBox[{"\"\<WikiDM\>\"", ",", "\"\<Names\>\""}], "]"}]}], "]"}]}], 
     ",", 
     RowBox[{"Axes", "\[Rule]", "False"}], ",", 
     RowBox[{"PlotRange", "\[Rule]", "All"}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", "1"}], ",", 
     RowBox[{"PlotStyle", "\[Rule]", 
      RowBox[{"PointSize", "[", "Large", "]"}]}]}], "]"}], ",", " ", 
   RowBox[{"ArrayPlot", "[", 
    RowBox[{
     RowBox[{"DMExample", "[", 
      RowBox[{"\"\<WikiDM\>\"", ",", "\"\<DistMat\>\""}], "]"}], ",", 
     RowBox[{"ColorFunction", "\[Rule]", "GrayLevel"}]}], "]"}]}], 
  "}"}]], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Distance-Matrix Fitting", "Section"],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
  "Fitting", " ", "of", " ", "vectors", " ", "with", " ", "an", " ", 
   "optimization", " ", "function"}], " ", "*)"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"DMFit", "[", 
   RowBox[{"distmat_", ",", "npd_", ",", 
    RowBox[{"distfunc_:", "EuclideanDistance"}]}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"ndmt", ",", "vars", ",", "p", ",", "vdmt", ",", "diff"}], "}"}],
     ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"ndmt", " ", "=", " ", 
      RowBox[{"Length", "[", "distmat", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"vars", " ", "=", " ", 
      RowBox[{"Array", "[", 
       RowBox[{"p", ",", 
        RowBox[{"{", 
         RowBox[{"ndmt", ",", "npd"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"vdmt", " ", "=", " ", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"distfunc", "[", 
         RowBox[{
          RowBox[{"vars", "[", 
           RowBox[{"[", "i1", "]"}], "]"}], ",", 
          RowBox[{"vars", "[", 
           RowBox[{"[", "i2", "]"}], "]"}]}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"i1", ",", "ndmt"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"i2", ",", "ndmt"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"diff", " ", "=", " ", 
      RowBox[{
       RowBox[{"Total", "[", 
        RowBox[{"Flatten", "[", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"distmat", " ", "-", " ", "vdmt"}], ")"}], "^", "2"}], 
         "]"}], "]"}], " ", "+", " ", 
       RowBox[{
        RowBox[{"Total", "[", 
         RowBox[{"Flatten", "[", 
          RowBox[{"distmat", "^", "2"}], "]"}], "]"}], "*", 
        RowBox[{
         RowBox[{"Total", "[", 
          RowBox[{"Flatten", "[", "vars", "]"}], "]"}], "^", "2"}]}]}]}], ";",
      "\[IndentingNewLine]", 
     RowBox[{"Partition", "[", 
      RowBox[{
       RowBox[{"NArgMin", "[", 
        RowBox[{"diff", ",", 
         RowBox[{"Flatten", "[", "vars", "]"}]}], "]"}], ",", "npd"}], 
      "]"}]}]}], "\[IndentingNewLine]", "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Multidimensional", " ", "scaling"}], " ", "*)"}]], "Input"],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
  "Square", " ", "of", " ", "Euclidean", " ", "distance", " ", "to", " ", 
   "inner", " ", "products", " ", "of", " ", "vectors"}], " ", 
  "*)"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"DMEucSqToDotProd", "[", "dmsq_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"dmsq1", ",", "dmsq2", ",", "dmsq0"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"dmsq1", " ", "=", " ", 
      RowBox[{"Mean", "[", "dmsq", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"dmsq2", " ", "=", " ", 
      RowBox[{"Mean", "[", 
       RowBox[{"Transpose", "[", "dmsq", "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"dmsq0", " ", "=", " ", 
      RowBox[{"Mean", "[", "dmsq1", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"(", 
        RowBox[{"1", "/", "2"}], ")"}]}], "*", 
      RowBox[{"Transpose", "[", 
       RowBox[{
        RowBox[{"Transpose", "[", 
         RowBox[{"dmsq", " ", "-", " ", "dmsq2"}], "]"}], " ", "-", " ", 
        "dmsq1", " ", "+", " ", "dmsq0"}], "]"}]}]}]}], "\[IndentingNewLine]",
    "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"DMSqMDS", "[", 
   RowBox[{"dmsq_", ",", "npd_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"dmdp", ",", "evals", ",", "evecs", ",", "crdlens"}], "}"}], ",",
     "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"dmdp", " ", "=", " ", 
      RowBox[{"DMEucSqToDotProd", "[", "dmsq", "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"evals", ",", "evecs"}], "}"}], " ", "=", " ", 
      RowBox[{"Eigensystem", "[", 
       RowBox[{"dmdp", ",", "npd"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"crdlens", " ", "=", " ", 
      RowBox[{"Sqrt", "[", 
       RowBox[{"Abs", "[", "evals", "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Transpose", "[", 
      RowBox[{"crdlens", "*", "evecs"}], "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"DMMDS", "[", 
   RowBox[{"dmsq_", ",", "npd_"}], "]"}], " ", ":=", " ", 
  RowBox[{"DMSqMDS", "[", 
   RowBox[{
    RowBox[{"dmsq", "^", "2"}], ",", "npd"}], "]"}]}]], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["K-Medoid Clustering (PAM)", "Section"],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"The", " ", "PAM", " ", "K"}], "-", 
    RowBox[{"Medoids", " ", "algorithm"}]}], ",", " ", 
   RowBox[{
   "here", " ", "working", " ", "from", " ", "a", " ", "distance", " ", 
    "matrix"}]}], " ", "*)"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"DMKMedoidMembership", "[", 
   RowBox[{"distmat_", ",", "mdixs_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "mddmat", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"mddmat", " ", "=", " ", 
      RowBox[{"Transpose", "[", 
       RowBox[{"distmat", "[", 
        RowBox[{"[", "mdixs", "]"}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{
       RowBox[{"First", "[", 
        RowBox[{"Ordering", "[", 
         RowBox[{"#", ",", "1"}], "]"}], "]"}], "&"}], " ", "/@", " ", 
      "mddmat"}]}]}], "\[IndentingNewLine]", "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"MembersToClusters", "[", "clusmems_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "ixcms", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"ixcms", " ", "=", " ", 
      RowBox[{"Transpose", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"Range", "[", 
          RowBox[{"Length", "[", "clusmems", "]"}], "]"}], ",", "clusmems"}], 
        "}"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{
       RowBox[{"#", "[", 
        RowBox[{"[", "1", "]"}], "]"}], "&"}], " ", "/@", " ", 
      RowBox[{"SortBy", "[", 
       RowBox[{
        RowBox[{"Transpose", "/@", " ", 
         RowBox[{"GatherBy", "[", 
          RowBox[{"ixcms", ",", 
           RowBox[{
            RowBox[{"#", "[", 
             RowBox[{"[", "2", "]"}], "]"}], "&"}]}], "]"}]}], ",", 
        RowBox[{
         RowBox[{"#", "[", 
          RowBox[{"[", 
           RowBox[{"2", ",", "1"}], "]"}], "]"}], "&"}]}], "]"}]}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"DMKMedoidImproveIndiv", "[", 
   RowBox[{"distmat_", ",", "cluster_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"clusdmat", ",", "clusdsums"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"clusdmat", " ", "=", " ", 
      RowBox[{"Transpose", "[", 
       RowBox[{
        RowBox[{"Transpose", "[", 
         RowBox[{"distmat", "[", 
          RowBox[{"[", "cluster", "]"}], "]"}], "]"}], "[", 
        RowBox[{"[", "cluster", "]"}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"clusdsums", " ", "=", " ", 
      RowBox[{"Total", " ", "/@", " ", "clusdmat"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"cluster", "[", 
      RowBox[{"[", 
       RowBox[{"First", "[", 
        RowBox[{"Ordering", "[", 
         RowBox[{"clusdsums", ",", "1"}], "]"}], "]"}], "]"}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"DMKMedoidImprove", "[", 
   RowBox[{"distmat_", ",", "clusters_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"DMKMedoidImproveIndiv", "[", 
     RowBox[{"distmat", ",", "#"}], "]"}], "&"}], " ", "/@", " ", 
   "clusters"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"DMKMedoids", "[", 
   RowBox[{"distmat_", ",", "initmdixs_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"mdixs", ",", "members", ",", "clusters", ",", "newmdixs"}], 
     "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"mdixs", " ", "=", " ", 
      RowBox[{"Union", "[", "initmdixs", "]"}]}], ";", "\[IndentingNewLine]", 
     
     RowBox[{"While", "[", 
      RowBox[{"True", ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"members", " ", "=", " ", 
         RowBox[{"DMKMedoidMembership", "[", 
          RowBox[{"distmat", ",", "mdixs"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"clusters", " ", "=", " ", 
         RowBox[{"MembersToClusters", "[", "members", "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"newmdixs", " ", "=", " ", 
         RowBox[{"Union", "[", 
          RowBox[{"DMKMedoidImprove", "[", 
           RowBox[{"distmat", ",", "clusters"}], "]"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"newmdixs", " ", "===", " ", "mdixs"}], ",", 
          RowBox[{"Break", "[", "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"mdixs", " ", "=", " ", "newmdixs"}]}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"mdixs", ",", "clusters", ",", "members"}], "}"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"RandomIndices", "[", 
   RowBox[{"n_", ",", "m_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Ordering", "[", 
   RowBox[{
    RowBox[{"RandomReal", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", "1"}], "}"}], ",", "n"}], "]"}], ",", "m"}], 
   "]"}]}]], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Neighbor Joining", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"NJTypes", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{"\"\<WPGMA\>\"", ",", "\"\<UPGMA\>\"", ",", "\"\<NeiSaitou\>\""}],
     "}"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"NeighborJoin", "::", "nottype"}], " ", "=", " ", 
   "\"\<`1` is not a recognized type\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"NeighborJoin", "[", 
   RowBox[{"distmat_", ",", "type_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"n", ",", "dinit", ",", "dfinal"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"!", 
        RowBox[{"MemberQ", "[", 
         RowBox[{"NJTypes", ",", "type"}], "]"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Message", "[", 
         RowBox[{
          RowBox[{"NeighborJoin", "::", "nottype"}], ",", "type"}], "]"}], 
        ";", "\[IndentingNewLine]", 
        RowBox[{"Return", "[", "]"}]}]}], "\[IndentingNewLine]", "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"!", 
        RowBox[{"CheckDistanceMatrix", "[", "distmat", "]"}]}], ",", " ", 
       RowBox[{"Return", "[", "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"n", " ", "=", " ", 
      RowBox[{"Length", "[", "distmat", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
       RowBox[{"Data", " ", "to", " ", 
        RowBox[{"transmit", ":", " ", 
         RowBox[{"branch", "-", 
          RowBox[{"node", " ", "index"}]}]}]}], ",", " ", 
       RowBox[{"distance", " ", "matrix"}], ",", " ", 
       RowBox[{"partial", " ", "trees"}], ",", " ", "weights"}], " ", "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"dinit", " ", "=", " ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"n", "+", "1"}], ",", "distmat", ",", " ", 
        RowBox[{"Range", "[", "n", "]"}], ",", " ", 
        RowBox[{"ConstantArray", "[", 
         RowBox[{"1", ",", "n"}], "]"}]}], "}"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"dfinal", " ", "=", " ", 
      RowBox[{"Nest", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"NJNext", "[", 
          RowBox[{"#", ",", "type"}], "]"}], "&"}], ",", "dinit", ",", 
        RowBox[{"n", "-", "1"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"NJAdjustBranchLengths", "[", 
      RowBox[{
       RowBox[{"dfinal", "[", 
        RowBox[{"[", 
         RowBox[{"3", ",", "1"}], "]"}], "]"}], ",", "type"}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"SmallestUprTriIxs", "[", "mat_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"i", ",", "j", ",", "n", ",", "ixs", ",", "val", ",", "nwval"}], 
     "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"n", " ", "=", " ", 
      RowBox[{"Length", "[", "mat", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"ixs", " ", "=", " ", 
      RowBox[{"{", 
       RowBox[{"1", ",", "2"}], "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"val", " ", "=", " ", 
      RowBox[{"mat", "[", 
       RowBox[{"[", 
        RowBox[{"1", ",", "2"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Do", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"nwval", " ", "=", " ", 
         RowBox[{"mat", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", "j"}], "]"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"nwval", " ", "<", " ", "val"}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"ixs", " ", "=", " ", 
            RowBox[{"{", 
             RowBox[{"i", ",", "j"}], "}"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"val", " ", "=", " ", "nwval"}]}]}], "\[IndentingNewLine]",
          "]"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"i", ",", 
         RowBox[{"n", "-", "1"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", 
         RowBox[{"i", "+", "1"}], ",", "n"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", "ixs"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"RemoveTwoAddOne", "[", 
   RowBox[{"xlst_", ",", "i_", ",", "j_", ",", "xnew_"}], "]"}], " ", ":=", 
  " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"d1", ",", "d2", ",", "d3", ",", "d4"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"d1", " ", "=", " ", 
      RowBox[{"xlst", "[", 
       RowBox[{"[", 
        RowBox[{"1", ";;", 
         RowBox[{"i", "-", "1"}]}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"d2", " ", "=", " ", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"j", "\[GreaterEqual]", 
         RowBox[{"i", "+", "2"}]}], ",", 
        RowBox[{"xlst", "[", 
         RowBox[{"[", 
          RowBox[{
           RowBox[{"i", "+", "1"}], ";;", 
           RowBox[{"j", "-", "1"}]}], "]"}], "]"}], ",", 
        RowBox[{"{", "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"d3", " ", "=", " ", 
      RowBox[{"xlst", "[", 
       RowBox[{"[", 
        RowBox[{
         RowBox[{"j", "+", "1"}], ";;"}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"d4", " ", "=", " ", 
      RowBox[{"{", "xnew", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Join", "[", 
      RowBox[{"d1", ",", "d2", ",", "d3", ",", "d4"}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"NJNext", "[", 
   RowBox[{"data_", ",", "type_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "ndix", ",", "distmat", ",", "leaves", ",", "weights", ",", "n", ",", 
      "i", ",", "j", ",", "dstij", ",", "wti", ",", "wtj", ",", "broff", ",", 
      "brlens", ",", "nwdst", ",", "nwlf", ",", "nwwt"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{"ndix", ",", "distmat", ",", "leaves", ",", "weights"}], "}"}],
       " ", "=", " ", "data"}], ";", "\[IndentingNewLine]", 
     RowBox[{"n", " ", "=", " ", 
      RowBox[{"Length", "[", "distmat", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"i", ",", "j"}], "}"}], " ", "=", " ", 
      RowBox[{"SmallestUprTriIxs", "[", 
       RowBox[{"NJDistMatForSelection", "[", 
        RowBox[{"distmat", ",", "type"}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"dstij", " ", "=", " ", 
      RowBox[{"distmat", "[", 
       RowBox[{"[", 
        RowBox[{"i", ",", "j"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"wti", " ", "=", " ", 
      RowBox[{"weights", "[", 
       RowBox[{"[", "i", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"wtj", " ", "=", " ", 
      RowBox[{"weights", "[", 
       RowBox[{"[", "j", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"broff", " ", "=", " ", 
      RowBox[{"NJBranchOffset", "[", 
       RowBox[{"n", ",", "type"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"broff", " ", "\[NotEqual]", " ", "0"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"broff", " ", "*=", " ", 
         RowBox[{
          RowBox[{"Total", "[", 
           RowBox[{"distmat", "[", 
            RowBox[{"[", "i", "]"}], "]"}], "]"}], " ", "-", " ", 
          RowBox[{"Total", "[", 
           RowBox[{"distmat", "[", 
            RowBox[{"[", "j", "]"}], "]"}], "]"}]}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"brlens", " ", "=", " ", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"dstij", "+", "broff"}], ",", 
            RowBox[{"dstij", "-", "broff"}]}], "}"}], "/", "2"}]}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"brlens", " ", "=", " ", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"dstij", ",", "dstij"}], "}"}], "/", "2"}]}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Do", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"nwdst", " ", "=", " ", 
         RowBox[{"NJDistMatNewRow", "[", 
          RowBox[{
           RowBox[{"distmat", "[", 
            RowBox[{"[", "i", "]"}], "]"}], ",", 
           RowBox[{"distmat", "[", 
            RowBox[{"[", "j", "]"}], "]"}], ",", "dstij", ",", "wti", ",", 
           "wtj", ",", "type"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"distmat", " ", "=", " ", 
         RowBox[{"Transpose", "[", 
          RowBox[{"RemoveTwoAddOne", "[", 
           RowBox[{"distmat", ",", "i", ",", "j", ",", "nwdst"}], "]"}], 
          "]"}]}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", "2", "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"distmat", " ", "*=", " ", 
      RowBox[{"(", 
       RowBox[{"1", " ", "-", " ", 
        RowBox[{"IdentityMatrix", "[", 
         RowBox[{"n", "-", "1"}], "]"}]}], ")"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"nwlf", " ", "=", " ", 
      RowBox[{"Join", "[", 
       RowBox[{
        RowBox[{"{", "ndix", "}"}], ",", "brlens", ",", 
        RowBox[{"leaves", "[", 
         RowBox[{"[", 
          RowBox[{"{", 
           RowBox[{"i", ",", "j"}], "}"}], "]"}], "]"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"leaves", " ", "=", " ", 
      RowBox[{"RemoveTwoAddOne", "[", 
       RowBox[{"leaves", ",", "i", ",", "j", ",", "nwlf"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"nwwt", " ", "=", " ", 
      RowBox[{"wti", " ", "+", " ", "wtj"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"weights", " ", "=", " ", 
      RowBox[{"RemoveTwoAddOne", "[", 
       RowBox[{"weights", ",", "i", ",", "j", ",", "nwwt"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"ndix", "+", "1"}], ",", "distmat", ",", "leaves", ",", " ", 
       "weights"}], "}"}]}]}], "\[IndentingNewLine]", "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"NJDistMatForSelection", "[", 
    RowBox[{"distmat_", ",", 
     RowBox[{"\"\<WPGMA\>\"", "|", "\"\<UPGMA\>\""}]}], "]"}], " ", ":=", " ",
    "distmat"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"NJDistMatForSelection", "[", 
   RowBox[{"distmat_", ",", "\"\<NeiSaitou\>\""}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"n", ",", "rowsums", ",", "colsums"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"n", " ", "=", " ", 
      RowBox[{"Length", "[", "distmat", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"rowsums", " ", "=", " ", 
      RowBox[{"Transpose", "[", 
       RowBox[{"ConstantArray", "[", 
        RowBox[{
         RowBox[{"Total", " ", "/@", " ", "distmat"}], ",", " ", "n"}], "]"}],
        "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"colsums", " ", "=", " ", 
      RowBox[{"ConstantArray", "[", 
       RowBox[{
        RowBox[{"Total", " ", "/@", " ", 
         RowBox[{"Transpose", "[", "distmat", "]"}]}], ",", "n"}], "]"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"n", " ", "-", " ", "2"}], ")"}], "*", "distmat"}], " ", "-", 
      " ", "rowsums", " ", "-", " ", "colsums"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"NJBranchOffset", "[", 
   RowBox[{"n_", ",", 
    RowBox[{"\"\<WPGMA\>\"", "|", "\"\<UPGMA\>\""}]}], "]"}], " ", ":=", " ", 
  "0"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"NJBranchOffset", "[", 
   RowBox[{"n_", ",", "\"\<NeiSaitou\>\""}], "]"}], " ", ":=", " ", 
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{"n", ">", "2"}], ",", 
    RowBox[{"1", "/", 
     RowBox[{"(", 
      RowBox[{"n", "-", "2"}], ")"}]}], ",", "0"}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"NJDistMatNewRow", "[", 
   RowBox[{
   "drow1_", ",", "drow2_", ",", "dst12_", ",", "wt1_", ",", "wt2_", ",", 
    "\"\<WPGMA\>\""}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"drow1", "+", "drow2"}], ")"}], "/", "2"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"NJDistMatNewRow", "[", 
   RowBox[{
   "drow1_", ",", "drow2_", ",", "dst12_", ",", "wt1_", ",", "wt2_", ",", 
    "\"\<UPGMA\>\""}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"wt1", "*", "drow1"}], "+", 
     RowBox[{"wt2", "*", "drow2"}]}], ")"}], "/", 
   RowBox[{"(", 
    RowBox[{"wt1", "+", "wt2"}], ")"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"NJDistMatNewRow", "[", 
   RowBox[{
   "drow1_", ",", "drow2_", ",", "dst12_", ",", "wt1_", ",", "wt2_", ",", 
    "\"\<NeiSaitou\>\""}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"drow1", "+", "drow2", " ", "-", " ", "dst12"}], ")"}], "/", 
   "2"}]}]], "Input"],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{"NJSubtractBranchLengths", "[", "tree_", "]"}], " ", ":=", " ", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "ndix", ",", "brlen1", ",", "brlen2", ",", "leaf1", ",", "leaf2"}], 
      "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Length", "[", "tree", "]"}], " ", "\[Equal]", " ", "0"}], 
        ",", 
        RowBox[{"Return", "[", "tree", "]"}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
        "ndix", ",", "brlen1", ",", "brlen2", ",", "leaf1", ",", "leaf2"}], 
        "}"}], " ", "=", " ", "tree"}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Length", "[", "leaf1", "]"}], " ", "\[NotEqual]", " ", 
         "0"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"brlen1", " ", "-=", " ", 
          RowBox[{"Mean", "[", 
           RowBox[{"leaf1", "[", 
            RowBox[{"[", 
             RowBox[{";;", "2"}], "]"}], "]"}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"leaf1", " ", "=", " ", 
          RowBox[{"NJSubtractBranchLengths", "[", "leaf1", "]"}]}]}]}], 
       "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Length", "[", "leaf2", "]"}], " ", "\[NotEqual]", " ", 
         "0"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"brlen2", " ", "-=", " ", 
          RowBox[{"Mean", "[", 
           RowBox[{"leaf2", "[", 
            RowBox[{"[", 
             RowBox[{";;", "2"}], "]"}], "]"}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"leaf2", " ", "=", " ", 
          RowBox[{"NJSubtractBranchLengths", "[", "leaf2", "]"}]}]}]}], 
       "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
       "ndix", ",", "brlen1", ",", "brlen2", ",", "leaf1", ",", "leaf2"}], 
       "}"}]}]}], "\[IndentingNewLine]", "]"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"NJAdjustBranchLengths", "[", 
   RowBox[{"tree_", ",", 
    RowBox[{"\"\<WPGMA\>\"", "|", "\"\<UPGMA\>\""}]}], "]"}], " ", ":=", " ", 
  
  RowBox[{"NJSubtractBranchLengths", "[", "tree", "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"NJAdjustBranchLengths", "[", 
   RowBox[{"tree_", ",", "\"\<NeiSaitou\>\""}], "]"}], " ", ":=", " ", 
  "tree"}]], "Input"],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{"How", " ", "many", " ", "leaf", " ", "nodes"}], ",", " ", 
   RowBox[{"from", " ", "traversing", " ", "the", " ", "tree"}]}], " ", 
  "*)"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"NJLeafCount", "[", "tree_", "]"}], " ", ":=", " ", 
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Length", "[", "tree", "]"}], " ", "\[NotEqual]", " ", "0"}], 
    ",", 
    RowBox[{
     RowBox[{"NJLeafCount", "[", 
      RowBox[{"tree", "[", 
       RowBox[{"[", "4", "]"}], "]"}], "]"}], "+", 
     RowBox[{"NJLeafCount", "[", 
      RowBox[{"tree", "[", 
       RowBox[{"[", "5", "]"}], "]"}], "]"}]}], ",", "1"}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{"Returns", " ", "root"}], "-", "to", "-", 
   RowBox[{"leaf", " ", "and", " ", "leaf"}], "-", "to", "-", 
   RowBox[{"leaf", " ", "distances"}]}], " ", "*)"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"NJPathLengths", "[", "tree_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "ndix", ",", "brlen1", ",", "brlen2", ",", "leaf1", ",", "leaf2", ",", 
      "prtlf1", ",", "prtlf2", ",", "plflf1", ",", "plflf2", ",", "prtlf", 
      ",", "plflf", ",", "prl1", ",", "prl2"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Length", "[", "tree", "]"}], " ", "\[NotEqual]", " ", "0"}], 
       ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"{", 
          RowBox[{
          "ndix", ",", "brlen1", ",", "brlen2", ",", "leaf1", ",", "leaf2"}], 
          "}"}], " ", "=", " ", "tree"}], ";", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"prtlf1", ",", "plflf1"}], "}"}], " ", "=", " ", 
         RowBox[{"NJPathLengths", "[", "leaf1", "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"prtlf2", ",", "plflf2"}], "}"}], " ", "=", " ", 
         RowBox[{"NJPathLengths", "[", "leaf2", "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"prtlf1", " ", "=", " ", 
         RowBox[{
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"#", "[", 
              RowBox[{"[", "1", "]"}], "]"}], ",", 
             RowBox[{
              RowBox[{"#", "[", 
               RowBox[{"[", "2", "]"}], "]"}], "+", "brlen1"}]}], "}"}], 
           "&"}], " ", "/@", " ", "prtlf1"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"prtlf2", " ", "=", " ", 
         RowBox[{
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"#", "[", 
              RowBox[{"[", "1", "]"}], "]"}], ",", 
             RowBox[{
              RowBox[{"#", "[", 
               RowBox[{"[", "2", "]"}], "]"}], "+", "brlen2"}]}], "}"}], 
           "&"}], " ", "/@", " ", "prtlf2"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"prtlf", " ", "=", " ", 
         RowBox[{"Join", "[", 
          RowBox[{"prtlf1", ",", "prtlf2"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"plflf", " ", "=", " ", 
         RowBox[{"Join", "[", 
          RowBox[{
           RowBox[{"Flatten", "[", 
            RowBox[{
             RowBox[{"Table", "[", 
              RowBox[{
               RowBox[{"Append", "[", 
                RowBox[{
                 RowBox[{"Sort", "[", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"prl1", "[", 
                    RowBox[{"[", "1", "]"}], "]"}], ",", 
                    RowBox[{"prl2", "[", 
                    RowBox[{"[", "1", "]"}], "]"}]}], "}"}], "]"}], ",", 
                 RowBox[{
                  RowBox[{"prl1", "[", 
                   RowBox[{"[", "2", "]"}], "]"}], "+", 
                  RowBox[{"prl2", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}]}], "]"}], ",", 
               RowBox[{"{", 
                RowBox[{"prl1", ",", "prtlf1"}], "}"}], ",", 
               RowBox[{"{", 
                RowBox[{"prl2", ",", "prtlf2"}], "}"}]}], "]"}], ",", "1"}], 
            "]"}], ",", "plflf1", ",", "plflf2"}], "]"}]}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"prtlf", " ", "=", " ", 
         RowBox[{"{", 
          RowBox[{"{", 
           RowBox[{"tree", ",", "0"}], "}"}], "}"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"plflf", " ", "=", " ", 
         RowBox[{"{", "}"}]}]}]}], "\[IndentingNewLine]", "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Sort", "[", "prtlf", "]"}], ",", 
       RowBox[{"Sort", "[", "plflf", "]"}]}], "}"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"NJMakeDistanceMatrix", "[", "tree_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"prtlf", ",", "plflf", ",", " ", "n", ",", "dmat"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{"prtlf", ",", "plflf"}], "}"}], " ", "=", " ", 
      RowBox[{"NJPathLengths", "[", "tree", "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"n", " ", "=", " ", 
      RowBox[{"Length", "[", "prtlf", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"dmat", " ", "=", " ", 
      RowBox[{"SparseArray", "[", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"#", "[", 
              RowBox[{"[", "1", "]"}], "]"}], ",", 
             RowBox[{"#", "[", 
              RowBox[{"[", "2", "]"}], "]"}]}], "}"}], "\[Rule]", 
           RowBox[{"#", "[", 
            RowBox[{"[", "3", "]"}], "]"}]}], "&"}], " ", "/@", " ", 
         "plflf"}], ",", " ", 
        RowBox[{"{", 
         RowBox[{"n", ",", "n"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"dmat", " ", "=", " ", 
      RowBox[{"Normal", "[", "dmat", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"dmat", " ", "+", " ", 
      RowBox[{"Transpose", "[", "dmat", "]"}]}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{"Flatten", " ", "the", " ", "tree"}], ",", " ", 
   RowBox[{
   "to", " ", "get", " ", "a", " ", "form", " ", "suitable", " ", "for", " ", 
    
    RowBox[{"Mma", "'"}], "s", " ", "graph", " ", "data", " ", "type"}]}], 
  " ", "*)"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"NJFlatten", "[", "tree_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"res", ",", "branches", ",", "n"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"res", " ", "=", " ", 
      RowBox[{"Reap", "[", 
       RowBox[{"NJFlattenNode", "[", "tree", "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"branches", " ", "=", " ", 
      RowBox[{"SortBy", "[", 
       RowBox[{
        RowBox[{"res", "[", 
         RowBox[{"[", 
          RowBox[{"2", ",", "1"}], "]"}], "]"}], ",", "First"}], "]"}]}], ";",
      "\[IndentingNewLine]", 
     RowBox[{"n", " ", "=", " ", 
      RowBox[{"NJLeafCount", "[", "tree", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"n", ",", "branches"}], "}"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"NJFlattenNode", "[", "tree_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "ndix", ",", "brlen1", ",", "brlen2", ",", "leaf1", ",", "leaf2", ",", 
      "lfix1", ",", "lfix2"}], "}"}], " ", ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Length", "[", "tree", "]"}], " ", "\[Equal]", " ", "0"}], 
       ",", "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{
        "No", " ", "change", " ", "if", " ", "at", " ", "a", " ", "leaf"}], 
        " ", "*)"}], "\[IndentingNewLine]", 
       RowBox[{"Return", "[", "]"}]}], "\[IndentingNewLine]", "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
       "ndix", ",", "brlen1", ",", "brlen2", ",", "leaf1", ",", "leaf2"}], 
       "}"}], " ", "=", " ", "tree"}], ";", "\[IndentingNewLine]", 
     RowBox[{"lfix1", " ", "=", " ", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Length", "[", "leaf1", "]"}], " ", "\[NotEqual]", " ", 
         "0"}], ",", 
        RowBox[{"leaf1", "[", 
         RowBox[{"[", "1", "]"}], "]"}], ",", "leaf1"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"lfix2", " ", "=", " ", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Length", "[", "leaf2", "]"}], " ", "\[NotEqual]", " ", 
         "0"}], ",", 
        RowBox[{"leaf2", "[", 
         RowBox[{"[", "1", "]"}], "]"}], ",", "leaf2"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Sow", "[", 
      RowBox[{"{", 
       RowBox[{
       "ndix", ",", "brlen1", ",", "brlen2", ",", "lfix1", ",", "lfix2"}], 
       "}"}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"NJFlattenNode", "[", "leaf1", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"NJFlattenNode", "[", "leaf2", "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"NJMakeMmaGraph", "[", "tree_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"n", ",", "branches"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{"n", ",", "branches"}], "}"}], " ", "=", " ", 
      RowBox[{"NJFlatten", "[", "tree", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Graph", "[", 
      RowBox[{
       RowBox[{"Flatten", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Labeled", "[", 
             RowBox[{
              RowBox[{"DirectedEdge", "[", 
               RowBox[{
                RowBox[{"#", "[", 
                 RowBox[{"[", "1", "]"}], "]"}], ",", 
                RowBox[{"#", "[", 
                 RowBox[{"[", "4", "]"}], "]"}]}], "]"}], ",", 
              RowBox[{"#", "[", 
               RowBox[{"[", "2", "]"}], "]"}]}], "]"}], ",", 
            RowBox[{"Labeled", "[", 
             RowBox[{
              RowBox[{"DirectedEdge", "[", 
               RowBox[{
                RowBox[{"#", "[", 
                 RowBox[{"[", "1", "]"}], "]"}], ",", 
                RowBox[{"#", "[", 
                 RowBox[{"[", "5", "]"}], "]"}]}], "]"}], ",", 
              RowBox[{"#", "[", 
               RowBox[{"[", "3", "]"}], "]"}]}], "]"}]}], "}"}], "&"}], " ", "/@",
          " ", "branches"}], "]"}], ",", 
       RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}]}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{"Doesn", "'"}], "t", " ", "do", " ", "any", " ", "validation"}], 
  " ", "*)"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"NJUnflatten", "[", "flattree_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "n", ",", "branches", ",", "isleafend", ",", "isbrncend", ",", "br", ",",
       "brtags", ",", "endbrs", ",", "midbrs", ",", "k", ",", "eblbl", ",", 
      "ix", ",", "ln1", ",", "ln2", ",", "lf1", ",", "lf2", ",", "lfe"}], 
     "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{"n", ",", "branches"}], "}"}], " ", "=", " ", "flattree"}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"isleafend", "[", "lf_", "]"}], " ", ":=", " ", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Length", "[", "lf", "]"}], "\[Equal]", "0"}], ",", 
        RowBox[{"lf", " ", "\[LessEqual]", " ", "n"}], ",", "True"}], "]"}]}],
      ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"isbrncend", "[", "brnc_", "]"}], " ", ":=", " ", 
      RowBox[{
       RowBox[{"isleafend", "[", 
        RowBox[{"brnc", "[", 
         RowBox[{"[", "4", "]"}], "]"}], "]"}], " ", "&&", " ", 
       RowBox[{"isleafend", "[", 
        RowBox[{"brnc", "[", 
         RowBox[{"[", "5", "]"}], "]"}], "]"}]}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{"True", ",", "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{"Makes", " ", "list", " ", "of", " ", 
         RowBox[{"{", 
          RowBox[{"branch", ",", " ", "tag"}], "}"}], " ", "where", " ", 
         "the", " ", "tags", " ", "are", " ", "whether", " ", "or", " ", 
         "not", " ", "no", " ", "branch", " ", "can", " ", "be", " ", 
         "placed", " ", "on", " ", "it"}], " ", "*)"}], "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"brtags", " ", "=", " ", 
         RowBox[{"isbrncend", " ", "/@", " ", "branches"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"endbrs", " ", "=", " ", 
         RowBox[{"Pick", "[", 
          RowBox[{"branches", ",", "brtags"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"midbrs", " ", "=", " ", 
         RowBox[{"Pick", "[", 
          RowBox[{"branches", ",", 
           RowBox[{"Not", " ", "/@", " ", "brtags"}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"Length", "[", "midbrs", "]"}], " ", "\[Equal]", " ", 
           "0"}], ",", 
          RowBox[{"Return", "[", 
           RowBox[{"First", "[", "endbrs", "]"}], "]"}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Do", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"eblbl", "[", "k", "]"}], " ", "=", " ", "Null"}], ",", 
          RowBox[{"{", 
           RowBox[{"k", ",", 
            RowBox[{"n", "+", "1"}], ",", 
            RowBox[{
             RowBox[{"2", "n"}], "-", "1"}]}], "}"}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Do", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"eblbl", "[", 
            RowBox[{"br", "[", 
             RowBox[{"[", "1", "]"}], "]"}], "]"}], " ", "=", " ", "br"}], 
          ",", 
          RowBox[{"{", 
           RowBox[{"br", ",", "endbrs"}], "}"}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"branches", " ", "=", " ", 
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"{", 
              RowBox[{"ix", ",", "ln1", ",", "ln2", ",", "lf1", ",", "lf2"}], 
              "}"}], " ", "=", " ", "br"}], ";", "\[IndentingNewLine]", 
            RowBox[{"{", 
             RowBox[{"ix", ",", " ", "ln1", ",", " ", "ln2", ",", 
              RowBox[{"If", "[", 
               RowBox[{
                RowBox[{"isleafend", "[", "lf1", "]"}], ",", "lf1", ",", 
                RowBox[{
                 RowBox[{"lfe", " ", "=", " ", 
                  RowBox[{"eblbl", "[", "lf1", "]"}]}], ";", " ", 
                 RowBox[{"If", "[", 
                  RowBox[{
                   RowBox[{"lfe", "===", "Null"}], ",", "lf1", ",", "lfe"}], 
                  "]"}]}]}], "]"}], ",", 
              RowBox[{"If", "[", 
               RowBox[{
                RowBox[{"isleafend", "[", "lf2", "]"}], ",", "lf2", ",", 
                RowBox[{
                 RowBox[{"lfe", " ", "=", " ", 
                  RowBox[{"eblbl", "[", "lf2", "]"}]}], ";", 
                 RowBox[{"If", "[", 
                  RowBox[{
                   RowBox[{"lfe", "===", "Null"}], ",", "lf2", ",", "lfe"}], 
                  "]"}]}]}], "]"}]}], "}"}]}], ",", "\[IndentingNewLine]", 
           RowBox[{"{", 
            RowBox[{"br", ",", "midbrs"}], "}"}]}], "]"}]}]}]}], 
      "\[IndentingNewLine]", "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
  "Dump", " ", "a", " ", "tree", " ", "constructed", " ", "by", " ", "this", 
   " ", "method"}], " ", "*)"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"NJDumpTree", "[", 
   RowBox[{"tree_", ",", "names_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "res", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"res", " ", "=", " ", 
      RowBox[{"Reap", "[", "\[IndentingNewLine]", 
       RowBox[{"NJDumpTreeNode", "[", 
        RowBox[{"tree", ",", "names", ",", "\"\<\>\""}], "]"}], 
       "\[IndentingNewLine]", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"StringJoin", "[", 
      RowBox[{"Riffle", "[", 
       RowBox[{
        RowBox[{"res", "[", 
         RowBox[{"[", 
          RowBox[{"2", ",", "1"}], "]"}], "]"}], ",", "\"\<\\n\>\""}], "]"}], 
      "]"}]}]}], "\[IndentingNewLine]", "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"NJDumpTree", "[", "tree_", "]"}], " ", ":=", " ", 
  RowBox[{"NJDumpTree", "[", 
   RowBox[{"tree", ",", 
    RowBox[{"Range", "[", 
     RowBox[{"NJLeafCount", "[", "tree", "]"}], "]"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"NJDumpTreePrefix", " ", "=", " ", "\"\<- \>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"NJDumpTreeNode", "[", 
   RowBox[{"node_", ",", "names_", ",", "prefix_"}], "]"}], " ", ":=", " ", 
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Length", "[", "node", "]"}], " ", "\[NotEqual]", " ", "0"}], 
    ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Sow", "[", 
      RowBox[{"prefix", " ", "<>", " ", 
       RowBox[{"ToString", "[", 
        RowBox[{"node", "[", 
         RowBox[{"[", "2", "]"}], "]"}], "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"NJDumpTreeNode", "[", 
      RowBox[{
       RowBox[{"node", "[", 
        RowBox[{"[", "4", "]"}], "]"}], ",", "names", ",", 
       RowBox[{"prefix", " ", "<>", "NJDumpTreePrefix"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Sow", "[", 
      RowBox[{"prefix", " ", "<>", " ", 
       RowBox[{"ToString", "[", 
        RowBox[{"node", "[", 
         RowBox[{"[", "3", "]"}], "]"}], "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"NJDumpTreeNode", "[", 
      RowBox[{
       RowBox[{"node", "[", 
        RowBox[{"[", "5", "]"}], "]"}], ",", "names", ",", 
       RowBox[{"prefix", " ", "<>", "NJDumpTreePrefix"}]}], "]"}]}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Sow", "[", 
      RowBox[{"prefix", " ", "<>", " ", 
       RowBox[{"ToString", "[", "node", "]"}], "<>", " ", "\"\<: \>\"", " ", "<>",
        " ", 
       RowBox[{"ToString", "[", 
        RowBox[{"names", "[", 
         RowBox[{"[", "node", "]"}], "]"}], "]"}]}], "]"}], ";"}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Fitch-Margoliash Tree Fitting", "Section"],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{
   "Returns", " ", "the", " ", "FM", " ", "tree", " ", "as", " ", "a", " ", 
    "list", " ", "of", " ", "links", " ", "with", " ", "format", " ", 
    RowBox[{"{", 
     RowBox[{"node", ",", "node"}], "}"}]}], ",", " ", 
   RowBox[{"and", " ", "gets", " ", "the", " ", "NJ", " ", 
    RowBox[{"tree", "'"}], "s", " ", "distances", " ", 
    RowBox[{"separately", "."}]}]}], " ", "*)"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"NJtoFMTree", "[", "njtree_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "n", ",", "njbrs", ",", "rpres", ",", "getlink1", ",", "getlink2", ",", 
      "getbothlinks", ",", "linksorig", ",", " ", "rootnode", ",", "withroot",
       ",", "withoutroot", ",", "othernode", ",", "rlnds", ",", "rllen", ",", 
      "links", ",", "dists"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{"n", ",", "njbrs"}], "}"}], " ", "=", " ", 
      RowBox[{"NJFlatten", "[", "njtree", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{"First", ",", " ", 
       RowBox[{"get", " ", "the", " ", "links"}]}], " ", "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"getlink1", "[", "br_", "]"}], " ", ":=", " ", 
      RowBox[{"Join", "[", 
       RowBox[{
        RowBox[{"Sort", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"br", "[", 
            RowBox[{"[", "1", "]"}], "]"}], ",", 
           RowBox[{"br", "[", 
            RowBox[{"[", "4", "]"}], "]"}]}], "}"}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"br", "[", 
          RowBox[{"[", "2", "]"}], "]"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"getlink2", "[", "br_", "]"}], " ", ":=", " ", 
      RowBox[{"Join", "[", 
       RowBox[{
        RowBox[{"Sort", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"br", "[", 
            RowBox[{"[", "1", "]"}], "]"}], ",", 
           RowBox[{"br", "[", 
            RowBox[{"[", "5", "]"}], "]"}]}], "}"}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"br", "[", 
          RowBox[{"[", "3", "]"}], "]"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"getbothlinks", "[", "br_", "]"}], " ", ":=", " ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"getlink1", "[", "br", "]"}], ",", 
        RowBox[{"getlink2", "[", "br", "]"}]}], "}"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"linksorig", " ", "=", " ", 
      RowBox[{"Flatten", "[", 
       RowBox[{
        RowBox[{"getbothlinks", " ", "/@", " ", "njbrs"}], ",", "1"}], 
       "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{"Find", " ", "the", " ", "root"}], " ", "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"rootnode", " ", "=", " ", 
      RowBox[{"First", "[", 
       RowBox[{"Complement", "[", 
        RowBox[{
         RowBox[{"Range", "[", 
          RowBox[{
           RowBox[{"2", "n"}], "-", "1"}], "]"}], ",", " ", 
         RowBox[{"Union", "[", 
          RowBox[{"Flatten", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"#", "[", 
              RowBox[{"[", 
               RowBox[{"4", ";;", "5"}], "]"}], "]"}], "&"}], " ", "/@", " ", 
            "njbrs"}], "]"}], "]"}]}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{"De", "-", 
       RowBox[{
       "root", " ", "the", " ", "tree", " ", "by", " ", "merging", " ", "the",
         " ", "links", " ", "with", " ", "the", " ", "root", " ", "node"}]}], 
      " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"withroot", " ", "=", " ", 
      RowBox[{"Select", "[", 
       RowBox[{"linksorig", ",", 
        RowBox[{
         RowBox[{"MemberQ", "[", 
          RowBox[{
           RowBox[{"#", "[", 
            RowBox[{"[", 
             RowBox[{"1", ";;", "2"}], "]"}], "]"}], ",", "rootnode"}], "]"}],
          "&"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"withoutroot", " ", "=", " ", 
      RowBox[{"Select", "[", 
       RowBox[{"linksorig", ",", 
        RowBox[{
         RowBox[{"!", 
          RowBox[{"MemberQ", "[", 
           RowBox[{
            RowBox[{"#", "[", 
             RowBox[{"[", 
              RowBox[{"1", ";;", "2"}], "]"}], "]"}], ",", "rootnode"}], 
           "]"}]}], "&"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"othernode", "[", "link_", "]"}], " ", ":=", " ", 
      RowBox[{"First", "[", 
       RowBox[{"Complement", "[", 
        RowBox[{
         RowBox[{"link", "[", 
          RowBox[{"[", 
           RowBox[{";;", "2"}], "]"}], "]"}], ",", 
         RowBox[{"{", "rootnode", "}"}]}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"rlnds", " ", "=", " ", 
      RowBox[{"Sort", "[", 
       RowBox[{"othernode", " ", "/@", " ", "withroot"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"rllen", " ", "=", " ", 
      RowBox[{"Total", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"#", "[", 
          RowBox[{"[", "3", "]"}], "]"}], "&"}], " ", "/@", " ", "withroot"}],
        "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"linksorig", " ", "=", " ", 
      RowBox[{"Sort", "[", 
       RowBox[{"Append", "[", 
        RowBox[{"withoutroot", ",", 
         RowBox[{"Append", "[", 
          RowBox[{"rlnds", ",", "rllen"}], "]"}]}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"links", " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"#", "[", 
         RowBox[{"[", 
          RowBox[{";;", "2"}], "]"}], "]"}], "&"}], " ", "/@", " ", 
       "linksorig"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"dists", " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"#", "[", 
         RowBox[{"[", "3", "]"}], "]"}], "&"}], " ", "/@", " ", 
       "linksorig"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"links", ",", "dists"}], "}"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{"Expands", " ", "out", " ", "a", " ", "tree"}], ",", " ", 
   RowBox[{
   "making", " ", "the", " ", "data", " ", "functions", " ", "of", " ", 
    "label", " ", "supplies", " ", "as", " ", "trxp"}]}], " ", 
  "*)"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FMTreeExpand", "::", "badtree"}], " ", "=", " ", 
   "\"\<the tree is invalid\>\""}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"FMTreeExpand", "[", 
   RowBox[{"tree_", ",", "trxp_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "dims", ",", "nlks", ",", "nodes", ",", "nd", ",", "ndlks", ",", "lk"}], 
     "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"dims", " ", "=", " ", 
      RowBox[{"Dimensions", "[", "tree", "]"}]}], ";", "\[IndentingNewLine]", 
     
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"dims", " ", "<", " ", "2"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Message", "[", 
         RowBox[{"FMTreeExpand", "::", "badtree"}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Return", "[", "False", "]"}]}]}], "\[IndentingNewLine]", 
      "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"dims", "[", 
         RowBox[{"[", "2", "]"}], "]"}], " ", "\[NotEqual]", " ", "2"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Message", "[", 
         RowBox[{"FMTreeExpand", "::", "badtree"}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Return", "[", "False", "]"}]}]}], "\[IndentingNewLine]", 
      "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Clear", "[", "trxp", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"nlks", " ", "=", " ", 
      RowBox[{"Length", "[", "tree", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"trxp", "[", "\"\<NumLinks\>\"", "]"}], " ", "=", " ", "nlks"}],
      ";", "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"trxp", "[", 
          RowBox[{"\"\<LinkNodes\>\"", ",", "k"}], "]"}], " ", "=", " ", 
         RowBox[{"tree", "[", 
          RowBox[{"[", "k", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"trxp", "[", 
          RowBox[{"\"\<LinkOther\>\"", ",", "k", ",", 
           RowBox[{"tree", "[", 
            RowBox[{"[", 
             RowBox[{"k", ",", "1"}], "]"}], "]"}]}], "]"}], " ", "=", " ", 
         RowBox[{"tree", "[", 
          RowBox[{"[", 
           RowBox[{"k", ",", "2"}], "]"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"trxp", "[", 
          RowBox[{"\"\<LinkOther\>\"", ",", "k", ",", 
           RowBox[{"tree", "[", 
            RowBox[{"[", 
             RowBox[{"k", ",", "2"}], "]"}], "]"}]}], "]"}], " ", "=", " ", 
         RowBox[{"tree", "[", 
          RowBox[{"[", 
           RowBox[{"k", ",", "1"}], "]"}], "]"}]}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"k", ",", "nlks"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"nodes", " ", "=", " ", 
      RowBox[{"Union", "[", 
       RowBox[{"Flatten", "[", "tree", "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"trxp", "[", "\"\<NodeList\>\"", "]"}], " ", "=", " ", 
      "nodes"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"trxp", "[", 
         RowBox[{"\"\<NodeLinks\>\"", ",", "nd"}], "]"}], " ", "=", " ", 
        RowBox[{"{", "}"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"nd", ",", "nodes"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"AppendTo", "[", 
         RowBox[{
          RowBox[{"trxp", "[", 
           RowBox[{"\"\<NodeLinks\>\"", ",", 
            RowBox[{"tree", "[", 
             RowBox[{"[", 
              RowBox[{"k", ",", "1"}], "]"}], "]"}]}], "]"}], ",", "k"}], 
         "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"AppendTo", "[", 
         RowBox[{
          RowBox[{"trxp", "[", 
           RowBox[{"\"\<NodeLinks\>\"", ",", 
            RowBox[{"tree", "[", 
             RowBox[{"[", 
              RowBox[{"k", ",", "2"}], "]"}], "]"}]}], "]"}], ",", "k"}], 
         "]"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"k", ",", "nlks"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"trxp", "[", 
         RowBox[{"\"\<NodeLinks\>\"", ",", "nd"}], "]"}], " ", "=", " ", 
        RowBox[{"Sort", "[", 
         RowBox[{"trxp", "[", 
          RowBox[{"\"\<NodeLinks\>\"", ",", "nd"}], "]"}], "]"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"nd", ",", "nodes"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"ndlks", " ", "=", " ", 
         RowBox[{"trxp", "[", 
          RowBox[{"\"\<NodeLinks\>\"", ",", "nd"}], "]"}]}], ";", 
        RowBox[{"Do", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"trxp", "[", 
            RowBox[{"\"\<NodeOther\>\"", ",", "nd", ",", "lk"}], "]"}], " ", 
           "=", " ", 
           RowBox[{"Complement", "[", 
            RowBox[{"ndlks", ",", 
             RowBox[{"{", "lk", "}"}]}], "]"}]}], ",", "\[IndentingNewLine]", 
          
          RowBox[{"{", 
           RowBox[{"lk", ",", "ndlks"}], "}"}]}], "]"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"nd", ",", "nodes"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", "True"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"FMMakeMmaGraph", "[", 
   RowBox[{"tree_", ",", "linklabels_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Graph", "[", 
   RowBox[{
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"Labeled", "[", 
       RowBox[{
        RowBox[{"UndirectedEdge", " ", "@@", " ", 
         RowBox[{"tree", "[", 
          RowBox[{"[", "k", "]"}], "]"}]}], ",", 
        RowBox[{"linklabels", "[", 
         RowBox[{"[", "k", "]"}], "]"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"k", ",", 
        RowBox[{"Length", "[", "tree", "]"}]}], "}"}]}], "]"}], ",", 
    RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"FMMakeMmaGraph", "[", "tree_", "]"}], " ", ":=", " ", 
  RowBox[{"FMMakeMmaGraph", "[", 
   RowBox[{"tree", ",", 
    RowBox[{"Range", "[", 
     RowBox[{"Length", "[", "tree", "]"}], "]"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Is", " ", "the", " ", "tree", " ", 
   RowBox[{"cyclic", "?", " ", "Returns"}], " ", "how", " ", "many", " ", 
   "cycles", " ", "were", " ", 
   RowBox[{"found", "."}]}], " ", "*)"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"FMCycles", "[", "tree_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"prunedtree", ",", "nd1", ",", "nd2", ",", "numcycles"}], "}"}], 
    ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"prunedtree", " ", "=", " ", "tree"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"numcycles", " ", "=", " ", "0"}], ";", "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Length", "[", "prunedtree", "]"}], " ", ">", " ", "0"}], ",",
        "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"{", 
          RowBox[{"nd1", ",", "nd2"}], "}"}], " ", "=", " ", 
         RowBox[{"First", "[", "prunedtree", "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"nd2", " ", "===", " ", "nd1"}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"numcycles", " ", "+=", " ", "1"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"prunedtree", " ", "=", " ", 
            RowBox[{"Rest", "[", "prunedtree", "]"}]}]}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"prunedtree", " ", "=", " ", 
           RowBox[{
            RowBox[{"Rest", "[", "prunedtree", "]"}], " ", "/.", " ", 
            RowBox[{"nd2", " ", "\[Rule]", " ", "nd1"}]}]}], ","}], 
         "\[IndentingNewLine]", "]"}]}]}], "\[IndentingNewLine]", "]"}], ";", 
     "\[IndentingNewLine]", "numcycles"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
  "Finds", " ", "all", " ", "paths", " ", "from", " ", "leaf", " ", "to", " ",
    "leaf", " ", "for", " ", "a", " ", "tree", " ", "if", " ", "it", " ", 
   "is", " ", "acyclic"}], " ", "*)"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"FMPaths", "[", "tree_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "trxp", ",", "nodes", ",", "nd", ",", "lks", ",", "lk", ",", "rpres"}], 
     "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"!", 
        RowBox[{"FMTreeExpand", "[", 
         RowBox[{"tree", ",", "trxp"}], "]"}]}], ",", 
       RowBox[{"Return", "[", "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"FMCycles", "[", "tree", "]"}], " ", ">", " ", "0"}], ",", 
       RowBox[{"Return", "[", "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"nodes", " ", "=", " ", 
      RowBox[{"trxp", "[", "\"\<NodeList\>\"", "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"rpres", " ", "=", " ", 
      RowBox[{"Reap", "[", "\[IndentingNewLine]", 
       RowBox[{"Do", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"lks", " ", "=", " ", 
           RowBox[{"trxp", "[", 
            RowBox[{"\"\<NodeLinks\>\"", ",", "nd"}], "]"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"Length", "[", "lks", "]"}], ">", "1"}], ",", 
            RowBox[{"Continue", "[", "]"}]}], "]"}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"lk", " ", "=", " ", 
           RowBox[{"First", "[", "lks", "]"}]}], ";", "\[IndentingNewLine]", 
          RowBox[{"FMPathSeg", "[", 
           RowBox[{"trxp", ",", "nd", ",", "lk", ",", 
            RowBox[{"{", "nd", "}"}]}], "]"}]}], ",", "\[IndentingNewLine]", 
         RowBox[{"{", 
          RowBox[{"nd", ",", "nodes"}], "}"}]}], "]"}], "\[IndentingNewLine]",
        "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"SortBy", "[", 
      RowBox[{
       RowBox[{"rpres", "[", 
        RowBox[{"[", 
         RowBox[{"2", ",", "1"}], "]"}], "]"}], ",", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"First", "[", "#", "]"}], ",", 
          RowBox[{"Last", "[", "#", "]"}]}], "}"}], "&"}]}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"FMPathSeg", "[", 
   RowBox[{"trxp_", ",", "nd_", ",", "lk_", ",", "path_"}], "]"}], " ", ":=", 
  " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"nwnd", ",", "nwpath", ",", "nwlks", ",", "nl"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"nwnd", " ", "=", " ", 
      RowBox[{"trxp", "[", 
       RowBox[{"\"\<LinkOther\>\"", ",", "lk", ",", "nd"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"nwpath", " ", "=", " ", 
      RowBox[{"Join", "[", 
       RowBox[{"path", ",", 
        RowBox[{"{", 
         RowBox[{"lk", ",", "nwnd"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"nwlks", " ", "=", " ", 
      RowBox[{"trxp", "[", 
       RowBox[{"\"\<NodeOther\>\"", ",", "nwnd", ",", "lk"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Length", "[", "nwlks", "]"}], "\[Equal]", "0"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Sow", "[", "nwpath", "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"Return", "[", "]"}]}]}], "\[IndentingNewLine]", "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{"FMPathSeg", "[", 
        RowBox[{"trxp", ",", "nwnd", ",", "nl", ",", "nwpath"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"nl", ",", "nwlks"}], "}"}]}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"FMPathMatrix", "[", "paths_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"path", ",", "nd1", ",", "nd2", ",", "lks", ",", "lk"}], "}"}], 
    ",", "\[IndentingNewLine]", 
    RowBox[{"SparseArray", "[", 
     RowBox[{"Flatten", "[", 
      RowBox[{
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"nd1", " ", "=", " ", 
           RowBox[{"First", "[", "path", "]"}]}], ";", " ", 
          RowBox[{"nd2", " ", "=", " ", 
           RowBox[{"Last", "[", "path", "]"}]}], ";", "\[IndentingNewLine]", 
          RowBox[{"lks", " ", "=", " ", 
           RowBox[{"First", " ", "/@", " ", 
            RowBox[{"Partition", "[", 
             RowBox[{
              RowBox[{"Rest", "[", "path", "]"}], ",", "2"}], "]"}]}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"Table", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"{", 
              RowBox[{"nd1", ",", "nd2", ",", "lk"}], "}"}], "\[Rule]", "1"}],
             ",", 
            RowBox[{"{", 
             RowBox[{"lk", ",", "lks"}], "}"}]}], "]"}]}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"{", 
          RowBox[{"path", ",", "paths"}], "}"}]}], "]"}], ",", "1"}], "]"}], 
     "]"}]}], "\[IndentingNewLine]", "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"FMDMCalc", "[", 
   RowBox[{"tree_", ",", "dists_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"FMPathMatrix", "[", 
    RowBox[{"FMPaths", "[", "tree", "]"}], "]"}], ".", "dists"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"FMDMCheckRMS", "[", 
   RowBox[{"tree_", ",", "dists_", ",", "distmat_"}], "]"}], " ", ":=", " ", 
  RowBox[{"RootMeanSquare", "[", 
   RowBox[{"Flatten", "[", 
    RowBox[{
     RowBox[{"FMDMCalc", "[", 
      RowBox[{"tree", ",", "dists"}], "]"}], " ", "-", " ", "distmat"}], 
    "]"}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"FMDMFit", "[", 
   RowBox[{"tree_", ",", "distmat_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "pathmat", ",", "n1", ",", "n2", ",", "nx", ",", "condmat", ",", "slvec",
       ",", "i1", ",", "i2", ",", "j1", ",", "j2", ",", "dists", ",", 
      "calcdm"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"pathmat", " ", "=", " ", 
      RowBox[{"FMPathMatrix", "[", 
       RowBox[{"FMPaths", "[", "tree", "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"n1", ",", "n2", ",", "nx"}], "}"}], " ", "=", " ", 
      RowBox[{"Dimensions", "[", "pathmat", "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"condmat", " ", "=", " ", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"Sum", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"pathmat", "[", 
            RowBox[{"[", 
             RowBox[{"j1", ",", "j2", ",", "i1"}], "]"}], "]"}], "*", 
           RowBox[{"pathmat", "[", 
            RowBox[{"[", 
             RowBox[{"j1", ",", "j2", ",", "i2"}], "]"}], "]"}]}], ",", 
          RowBox[{"{", 
           RowBox[{"j1", ",", "n1"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"j2", ",", "n2"}], "}"}]}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"i1", ",", "nx"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"i2", ",", "nx"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"slvec", " ", "=", " ", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"Sum", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"pathmat", "[", 
            RowBox[{"[", 
             RowBox[{"j1", ",", "j2", ",", "i1"}], "]"}], "]"}], "*", 
           RowBox[{"distmat", "[", 
            RowBox[{"[", 
             RowBox[{"j1", ",", "j2"}], "]"}], "]"}]}], ",", 
          RowBox[{"{", 
           RowBox[{"j1", ",", "n1"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"j2", ",", "n2"}], "}"}]}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"i1", ",", "nx"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"dists", " ", "=", " ", 
      RowBox[{"LinearSolve", "[", 
       RowBox[{"condmat", ",", "slvec"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"calcdm", " ", "=", " ", 
      RowBox[{"pathmat", ".", "dists"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"dists", ",", 
       RowBox[{"RootMeanSquare", "[", 
        RowBox[{"Flatten", "[", 
         RowBox[{"calcdm", "-", "distmat"}], "]"}], "]"}]}], "}"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Fitch-Margoliash Tree Building", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"FMAddToTree", "[", 
   RowBox[{"tree_", ",", "newleaf_", ",", "newbranch_", ",", "oldlink_"}], 
   "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"nd1", ",", "nd2"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{"nd1", ",", "nd2"}], "}"}], " ", "=", " ", 
      RowBox[{"tree", "[", 
       RowBox[{"[", "oldlink", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Join", "[", 
      RowBox[{
       RowBox[{"Delete", "[", 
        RowBox[{"tree", ",", "oldlink"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"newleaf", ",", "newbranch"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"nd1", ",", "newbranch"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"nd2", ",", "newbranch"}], "}"}]}], "}"}]}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Uses", " ", 
   RowBox[{"RandomReal", "[", "]"}], " ", "and", " ", 
   RowBox[{"RandomInteger", "[", "]"}]}], " ", "*)"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"FMRandomTree", "[", "n_Integer", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"leaves", ",", "tree", ",", "k", ",", "oldlink"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"n", "\[LessEqual]", "1"}], ",", 
       RowBox[{"Return", "[", "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"leaves", " ", "=", " ", 
      RowBox[{"Ordering", "[", 
       RowBox[{"RandomReal", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"0", ",", "1"}], "}"}], ",", "n"}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"tree", " ", "=", " ", 
      RowBox[{"{", 
       RowBox[{"leaves", "[", 
        RowBox[{"[", 
         RowBox[{"{", 
          RowBox[{"1", ",", "2"}], "}"}], "]"}], "]"}], "}"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"oldlink", " ", "=", " ", 
         RowBox[{"RandomInteger", "[", 
          RowBox[{"{", 
           RowBox[{"1", ",", 
            RowBox[{"Length", "[", "tree", "]"}]}], "}"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"tree", " ", "=", " ", 
         RowBox[{"FMAddToTree", "[", 
          RowBox[{"tree", ",", 
           RowBox[{"leaves", "[", 
            RowBox[{"[", "k", "]"}], "]"}], ",", 
           RowBox[{"n", "+", "k", "-", "2"}], ",", "oldlink"}], "]"}]}]}], 
       ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"k", ",", "3", ",", "n"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", "tree"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{"Create", " ", "a", " ", "tree", " ", "incrementally"}], ",", " ", 
   
   RowBox[{
   "finding", " ", "the", " ", "best", " ", "fit", " ", "as", " ", "one", " ",
     "goes"}]}], " ", "*)"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"FMIncremental", "[", "distmat_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "n", ",", "tree", ",", "fit", ",", "indxs", ",", "bestitf", ",", "i1", 
      ",", "i2", ",", "i3", ",", "i4", ",", "dmixs", ",", "dm", ",", "oldixs",
       ",", "oldtree", ",", "oldfit", ",", "k", ",", "ixcs", ",", "nwix", ",",
       "l"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"n", " ", "=", " ", 
      RowBox[{"Length", "[", "distmat", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{"Special", " ", "cases"}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"Which", "[", 
      RowBox[{
       RowBox[{"n", "\[LessEqual]", "1"}], ",", 
       RowBox[{"Return", "[", "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"n", " ", "\[Equal]", " ", "2"}], ",", " ", 
       RowBox[{
        RowBox[{"tree", " ", "=", " ", 
         RowBox[{"{", 
          RowBox[{"{", 
           RowBox[{"1", ",", "2"}], "}"}], "}"}]}], ";", 
        "\[IndentingNewLine]", " ", 
        RowBox[{"fit", " ", "=", " ", 
         RowBox[{"FMDMFit", "[", 
          RowBox[{"tree", ",", "distmat"}], "]"}]}], ";", " ", 
        RowBox[{"Return", "[", 
         RowBox[{"{", 
          RowBox[{"tree", ",", "fit"}], "}"}], "]"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"n", " ", "\[Equal]", " ", "3"}], ",", " ", 
       RowBox[{
        RowBox[{"tree", " ", "=", " ", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"1", ",", "4"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"2", ",", "4"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"3", ",", "4"}], "}"}]}], "}"}]}], ";", 
        "\[IndentingNewLine]", " ", 
        RowBox[{"fit", " ", "=", " ", 
         RowBox[{"FMDMFit", "[", 
          RowBox[{"tree", ",", "distmat"}], "]"}]}], ";", " ", 
        RowBox[{"Return", "[", 
         RowBox[{"{", 
          RowBox[{"tree", ",", "fit"}], "}"}], "]"}]}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
       RowBox[{"Find", " ", "the", " ", "best"}], "-", 
       RowBox[{
       "fitting", " ", "subtree", " ", "with", " ", "four", " ", "points"}]}],
       " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"dmixs", "[", "ixs_", "]"}], " ", ":=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"#", "[", 
         RowBox[{"[", "ixs", "]"}], "]"}], "&"}], " ", "/@", " ", 
       RowBox[{"distmat", "[", 
        RowBox[{"[", "ixs", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"bestitf", " ", "=", " ", "Null"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Do", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"indxs", " ", "=", " ", 
         RowBox[{"{", 
          RowBox[{"i1", ",", "i2", ",", "i3", ",", "i4"}], "}"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"dm", " ", "=", " ", 
         RowBox[{"dmixs", "[", "indxs", "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"tree", " ", "=", " ", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"1", ",", 
             RowBox[{"n", "+", "1"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"2", ",", 
             RowBox[{"n", "+", "1"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"3", ",", 
             RowBox[{"n", "+", "2"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"4", ",", 
             RowBox[{"n", "+", "2"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"n", "+", "1"}], ",", 
             RowBox[{"n", "+", "2"}]}], "}"}]}], "}"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"fit", " ", "=", " ", 
         RowBox[{"FMDMFit", "[", 
          RowBox[{"tree", ",", "dm"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"bestitf", " ", "===", " ", "Null"}], ",", " ", 
          RowBox[{"bestitf", " ", "=", " ", 
           RowBox[{"{", 
            RowBox[{"indxs", ",", "tree", ",", "fit"}], "}"}]}]}], "]"}], ";",
         "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"fit", "[", 
            RowBox[{"[", "2", "]"}], "]"}], " ", "<", " ", 
           RowBox[{"bestitf", "[", 
            RowBox[{"[", 
             RowBox[{"3", ",", "2"}], "]"}], "]"}]}], ",", 
          RowBox[{"bestitf", " ", "=", " ", 
           RowBox[{"{", 
            RowBox[{"indxs", ",", "tree", ",", "fit"}], "}"}]}]}], "]"}], ";",
         "\[IndentingNewLine]", 
        RowBox[{"tree", " ", "=", " ", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"1", ",", 
             RowBox[{"n", "+", "1"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"3", ",", 
             RowBox[{"n", "+", "1"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"2", ",", 
             RowBox[{"n", "+", "2"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"4", ",", 
             RowBox[{"n", "+", "2"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"n", "+", "1"}], ",", 
             RowBox[{"n", "+", "2"}]}], "}"}]}], "}"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"fit", " ", "=", " ", 
         RowBox[{"FMDMFit", "[", 
          RowBox[{"tree", ",", "dm"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"bestitf", " ", "===", " ", "Null"}], ",", " ", 
          RowBox[{"bestitf", " ", "=", " ", 
           RowBox[{"{", 
            RowBox[{"indxs", ",", "tree", ",", "fit"}], "}"}]}]}], "]"}], ";",
         "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"fit", "[", 
            RowBox[{"[", "2", "]"}], "]"}], " ", "<", " ", 
           RowBox[{"bestitf", "[", 
            RowBox[{"[", 
             RowBox[{"3", ",", "2"}], "]"}], "]"}]}], ",", 
          RowBox[{"bestitf", " ", "=", " ", 
           RowBox[{"{", 
            RowBox[{"indxs", ",", "tree", ",", "fit"}], "}"}]}]}], "]"}], ";",
         "\[IndentingNewLine]", 
        RowBox[{"tree", " ", "=", " ", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"1", ",", 
             RowBox[{"n", "+", "1"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"4", ",", 
             RowBox[{"n", "+", "1"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"2", ",", 
             RowBox[{"n", "+", "2"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"3", ",", 
             RowBox[{"n", "+", "2"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"n", "+", "1"}], ",", 
             RowBox[{"n", "+", "2"}]}], "}"}]}], "}"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"fit", " ", "=", " ", 
         RowBox[{"FMDMFit", "[", 
          RowBox[{"tree", ",", "dm"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"bestitf", " ", "===", " ", "Null"}], ",", " ", 
          RowBox[{"bestitf", " ", "=", " ", 
           RowBox[{"{", 
            RowBox[{"indxs", ",", "tree", ",", "fit"}], "}"}]}]}], "]"}], ";",
         "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"fit", "[", 
            RowBox[{"[", "2", "]"}], "]"}], " ", "<", " ", 
           RowBox[{"bestitf", "[", 
            RowBox[{"[", 
             RowBox[{"3", ",", "2"}], "]"}], "]"}]}], ",", 
          RowBox[{"bestitf", " ", "=", " ", 
           RowBox[{"{", 
            RowBox[{"indxs", ",", "tree", ",", "fit"}], "}"}]}]}], "]"}]}], 
       ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"i1", ",", "n"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"i2", ",", 
         RowBox[{"i1", "+", "1"}], ",", "n"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"i3", ",", 
         RowBox[{"i2", "+", "1"}], ",", "n"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"i4", ",", 
         RowBox[{"i3", "+", "1"}], ",", "n"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
      "Do", " ", "that", " ", "for", " ", "five", " ", "or", " ", "more", " ",
        "points"}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"Do", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"{", 
          RowBox[{"oldixs", ",", "oldtree", ",", "oldfit"}], "}"}], " ", "=", 
         " ", "bestitf"}], ";", "\[IndentingNewLine]", 
        RowBox[{"ixcs", " ", "=", " ", 
         RowBox[{"Complement", "[", 
          RowBox[{
           RowBox[{"Range", "[", "n", "]"}], ",", "oldixs"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"bestitf", " ", "=", " ", "Null"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Do", "[", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{"indxs", " ", "=", " ", 
            RowBox[{"Append", "[", 
             RowBox[{"oldixs", ",", "nwix"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"dm", " ", "=", " ", 
            RowBox[{"dmixs", "[", "indxs", "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"Do", "[", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{
              RowBox[{"tree", " ", "=", " ", 
               RowBox[{"FMAddToTree", "[", 
                RowBox[{"oldtree", ",", "k", ",", 
                 RowBox[{"n", "+", "k", "-", "2"}], ",", "l"}], "]"}]}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{"fit", " ", "=", " ", 
               RowBox[{"FMDMFit", "[", 
                RowBox[{"tree", ",", "dm"}], "]"}]}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{"If", "[", 
               RowBox[{
                RowBox[{"bestitf", " ", "===", " ", "Null"}], ",", " ", 
                RowBox[{"bestitf", " ", "=", " ", 
                 RowBox[{"{", 
                  RowBox[{"indxs", ",", "tree", ",", "fit"}], "}"}]}]}], 
               "]"}], ";", "\[IndentingNewLine]", 
              RowBox[{"If", "[", 
               RowBox[{
                RowBox[{
                 RowBox[{"fit", "[", 
                  RowBox[{"[", "2", "]"}], "]"}], " ", "<", " ", 
                 RowBox[{"bestitf", "[", 
                  RowBox[{"[", 
                   RowBox[{"3", ",", "2"}], "]"}], "]"}]}], ",", 
                RowBox[{"bestitf", " ", "=", " ", 
                 RowBox[{"{", 
                  RowBox[{"indxs", ",", "tree", ",", "fit"}], "}"}]}]}], 
               "]"}]}], ",", "\[IndentingNewLine]", 
             RowBox[{"{", 
              RowBox[{"l", ",", 
               RowBox[{"Length", "[", "oldtree", "]"}]}], "}"}]}], "]"}]}], 
          ",", "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"nwix", ",", "ixcs"}], "}"}]}], "]"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"k", ",", "5", ",", "n"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"tree", " ", "=", " ", 
      RowBox[{
       RowBox[{"bestitf", "[", 
        RowBox[{"[", "2", "]"}], "]"}], " ", "/.", " ", 
       RowBox[{"Thread", "[", 
        RowBox[{
         RowBox[{"Range", "[", "n", "]"}], "\[Rule]", 
         RowBox[{"bestitf", "[", 
          RowBox[{"[", "1", "]"}], "]"}]}], "]"}]}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"tree", " ", "=", " ", 
      RowBox[{"Sort", "[", 
       RowBox[{"Sort", " ", "/@", " ", "tree"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"fit", " ", "=", " ", 
      RowBox[{"FMDMFit", "[", 
       RowBox[{"tree", ",", "distmat"}], "]"}]}], ";", "\[IndentingNewLine]", 
     
     RowBox[{"{", 
      RowBox[{"tree", ",", "fit"}], "}"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"FMImproveFlipStep", "[", 
   RowBox[{"tree_", ",", "distmat_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "nwtree", ",", "fit", ",", "nwfit", ",", "isimproved", ",", "trxp", ",", 
      "nl", ",", "l", ",", "nd1", ",", "nd2", ",", "olks1", ",", "olks2", ",",
       "ndl11", ",", "ndl12", ",", "ndl21", ",", "ndl22", ",", "bstree", ",", 
      "trtree", ",", "trfit"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"fit", " ", "=", " ", 
      RowBox[{"FMDMFit", "[", 
       RowBox[{"tree", ",", "distmat"}], "]"}]}], ";", "\[IndentingNewLine]", 
     
     RowBox[{"nwtree", " ", "=", " ", "tree"}], ";", "\[IndentingNewLine]", 
     RowBox[{"nwfit", " ", "=", " ", "fit"}], ";", "\[IndentingNewLine]", 
     RowBox[{"isimproved", " ", "=", " ", "False"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"FMTreeExpand", "[", 
      RowBox[{"tree", ",", "trxp"}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"nl", " ", "=", " ", 
      RowBox[{"trxp", "[", "\"\<NumLinks\>\"", "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"{", 
          RowBox[{"nd1", ",", "nd2"}], "}"}], " ", "=", " ", 
         RowBox[{"tree", "[", 
          RowBox[{"[", "l", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"olks1", " ", "=", " ", 
         RowBox[{"trxp", "[", 
          RowBox[{"\"\<NodeOther\>\"", ",", "nd1", ",", "l"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"Length", "[", "olks1", "]"}], " ", "<", " ", "2"}], ",", 
          " ", 
          RowBox[{"Continue", "[", "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"olks2", " ", "=", " ", 
         RowBox[{"trxp", "[", 
          RowBox[{"\"\<NodeOther\>\"", ",", "nd2", ",", "l"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"Length", "[", "olks1", "]"}], " ", "<", " ", "2"}], ",", 
          " ", 
          RowBox[{"Continue", "[", "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"ndl11", " ", "=", " ", 
         RowBox[{"trxp", "[", 
          RowBox[{"\"\<LinkOther\>\"", ",", 
           RowBox[{"olks1", "[", 
            RowBox[{"[", "1", "]"}], "]"}], ",", "nd1"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"ndl12", " ", "=", " ", 
         RowBox[{"trxp", "[", 
          RowBox[{"\"\<LinkOther\>\"", ",", 
           RowBox[{"olks1", "[", 
            RowBox[{"[", "2", "]"}], "]"}], ",", "nd1"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"ndl21", " ", "=", " ", 
         RowBox[{"trxp", "[", 
          RowBox[{"\"\<LinkOther\>\"", ",", 
           RowBox[{"olks2", "[", 
            RowBox[{"[", "1", "]"}], "]"}], ",", "nd2"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"ndl22", " ", "=", " ", 
         RowBox[{"trxp", "[", 
          RowBox[{"\"\<LinkOther\>\"", ",", 
           RowBox[{"olks2", "[", 
            RowBox[{"[", "2", "]"}], "]"}], ",", "nd2"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"(*", " ", 
         RowBox[{
         "Base", " ", "tree", " ", "for", " ", "the", " ", "trial", " ", 
          "trees"}], " ", "*)"}], "\[IndentingNewLine]", 
        RowBox[{"bstree", " ", "=", " ", 
         RowBox[{"Delete", "[", 
          RowBox[{"tree", ",", 
           RowBox[{"Transpose", "[", 
            RowBox[{"{", 
             RowBox[{"Join", "[", 
              RowBox[{"olks1", ",", "olks2"}], "]"}], "}"}], "]"}]}], "]"}]}],
         ";", "\[IndentingNewLine]", 
        RowBox[{"(*", " ", 
         RowBox[{"Trial", " ", "tree", " ", "1"}], " ", "*)"}], 
        "\[IndentingNewLine]", 
        RowBox[{"trtree", " ", "=", " ", 
         RowBox[{"Join", "[", 
          RowBox[{"bstree", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"ndl11", ",", "nd1"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"ndl21", ",", "nd1"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"ndl12", ",", "nd2"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"ndl22", ",", "nd2"}], "}"}]}], "}"}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"trfit", " ", "=", " ", 
         RowBox[{"FMDMFit", "[", 
          RowBox[{"trtree", ",", "distmat"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"trfit", "[", 
            RowBox[{"[", "2", "]"}], "]"}], " ", "<", " ", 
           RowBox[{"nwfit", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"isimproved", " ", "=", " ", "True"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"nwtree", " ", "=", " ", "trtree"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"nwfit", " ", "=", " ", "trfit"}]}]}], 
         "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"(*", " ", 
         RowBox[{"Trial", " ", "tree", " ", "2"}], " ", "*)"}], 
        "\[IndentingNewLine]", 
        RowBox[{"trtree", " ", "=", " ", 
         RowBox[{"Join", "[", 
          RowBox[{"bstree", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"ndl11", ",", "nd1"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"ndl22", ",", "nd1"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"ndl21", ",", "nd2"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"ndl12", ",", "nd2"}], "}"}]}], "}"}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"trfit", " ", "=", " ", 
         RowBox[{"FMDMFit", "[", 
          RowBox[{"trtree", ",", "distmat"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"trfit", "[", 
            RowBox[{"[", "2", "]"}], "]"}], " ", "<", " ", 
           RowBox[{"nwfit", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"isimproved", " ", "=", " ", "True"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"nwtree", " ", "=", " ", "trtree"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"nwfit", " ", "=", " ", "trfit"}]}]}], 
         "\[IndentingNewLine]", "]"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"l", ",", "nl"}], "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
     
     RowBox[{"{", 
      RowBox[{"isimproved", ",", "nwtree", ",", "nwfit"}], "}"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Hill", " ", 
   RowBox[{"climing", " ", "--"}], " ", "could", " ", "improve", " ", "with", 
   " ", "simulated", " ", "annealing"}], " ", "*)"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"FMImproveByFlipping", "[", 
   RowBox[{"tree_", ",", "distmat_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"isimproved", ",", "nwtree", ",", "nwfit"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"nwtree", " ", "=", " ", "tree"}], ";", "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{"True", ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"{", 
          RowBox[{"isimproved", ",", "nwtree", ",", "nwfit"}], "}"}], " ", 
         "=", " ", 
         RowBox[{"FMImproveFlipStep", "[", 
          RowBox[{"nwtree", ",", "distmat"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"!", "isimproved"}], ",", 
          RowBox[{"Break", "[", "]"}]}], "]"}]}]}], "\[IndentingNewLine]", 
      "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"nwtree", " ", "=", " ", 
      RowBox[{"Sort", "[", 
       RowBox[{"Sort", " ", "/@", " ", "nwtree"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"nwfit", " ", "=", " ", 
      RowBox[{"FMDMFit", "[", 
       RowBox[{"nwtree", ",", "distmat"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"nwtree", ",", "nwfit"}], "}"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input"]
}, Closed]]
},
WindowSize->{740, 696},
WindowMargins->{{142, Automatic}, {Automatic, 8}},
FrontEndVersion->"11.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (September \
21, 2016)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[557, 20, 6220, 275, 4818, "Text"],
Cell[CellGroupData[{
Cell[6802, 299, 54, 0, 64, "Section"],
Cell[6859, 301, 428, 12, 54, "Input"],
Cell[7290, 315, 171, 4, 32, "Input"],
Cell[7464, 321, 176, 4, 32, "Input"],
Cell[7643, 327, 162, 4, 32, "Input"],
Cell[7808, 333, 178, 4, 32, "Input"],
Cell[7989, 339, 2829, 72, 432, "Input"],
Cell[10821, 413, 294, 8, 32, "Input"],
Cell[11118, 423, 680, 17, 75, "Input"],
Cell[11801, 442, 294, 8, 32, "Input"],
Cell[12098, 452, 662, 17, 75, "Input"],
Cell[12763, 471, 312, 8, 32, "Input"],
Cell[13078, 481, 589, 19, 54, "Input"],
Cell[13670, 502, 930, 28, 96, "Input"],
Cell[14603, 532, 733, 21, 117, "Input"],
Cell[15339, 555, 952, 25, 117, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[16328, 585, 42, 0, 50, "Section"],
Cell[16373, 587, 176, 4, 32, "Input"],
Cell[16552, 593, 1972, 54, 201, "Input"],
Cell[18527, 649, 105, 2, 32, "Input"],
Cell[18635, 653, 204, 5, 32, "Input"],
Cell[18842, 660, 997, 26, 138, "Input"],
Cell[19842, 688, 906, 24, 138, "Input"],
Cell[20751, 714, 211, 6, 32, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[20999, 725, 44, 0, 50, "Section"],
Cell[21046, 727, 288, 8, 54, "Input"],
Cell[21337, 737, 661, 18, 96, "Input"],
Cell[22001, 757, 1063, 30, 96, "Input"],
Cell[23067, 789, 961, 26, 117, "Input"],
Cell[24031, 817, 288, 8, 54, "Input"],
Cell[24322, 827, 1518, 36, 243, "Input"],
Cell[25843, 865, 309, 10, 32, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[26189, 880, 35, 0, 50, "Section"],
Cell[26227, 882, 187, 5, 32, "Input"],
Cell[26417, 889, 158, 4, 32, "Input"],
Cell[26578, 895, 2267, 59, 285, "Input"],
Cell[28848, 956, 1654, 42, 285, "Input"],
Cell[30505, 1000, 1340, 38, 159, "Input"],
Cell[31848, 1040, 4547, 110, 558, "Input"],
Cell[36398, 1152, 219, 6, 34, "Input"],
Cell[36620, 1160, 1141, 29, 138, "Input"],
Cell[37764, 1191, 178, 5, 34, "Input"],
Cell[37945, 1198, 308, 9, 32, "Input"],
Cell[38256, 1209, 288, 8, 54, "Input"],
Cell[38547, 1219, 397, 12, 54, "Input"],
Cell[38947, 1233, 320, 9, 54, "Input"],
Cell[39270, 1244, 2150, 56, 285, "Input"],
Cell[41423, 1302, 241, 6, 34, "Input"],
Cell[41667, 1310, 158, 4, 32, "Input"],
Cell[41828, 1316, 209, 5, 32, "Input"],
Cell[42040, 1323, 487, 14, 54, "Input"],
Cell[42530, 1339, 226, 5, 32, "Input"],
Cell[42759, 1346, 3848, 99, 411, "Input"],
Cell[46610, 1447, 1425, 38, 159, "Input"],
Cell[48038, 1487, 302, 8, 54, "Input"],
Cell[48343, 1497, 879, 24, 138, "Input"],
Cell[49225, 1523, 1971, 51, 285, "Input"],
Cell[51199, 1576, 1563, 41, 180, "Input"],
Cell[52765, 1619, 147, 4, 32, "Input"],
Cell[52915, 1625, 4833, 117, 474, "Input"],
Cell[57751, 1744, 166, 4, 32, "Input"],
Cell[57920, 1750, 753, 19, 138, "Input"],
Cell[58676, 1771, 243, 6, 32, "Input"],
Cell[58922, 1779, 101, 2, 32, "Input"],
Cell[59026, 1783, 1590, 42, 159, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[60653, 1830, 48, 0, 50, "Section"],
Cell[60704, 1832, 455, 10, 54, "Input"],
Cell[61162, 1844, 5649, 151, 495, "Input"],
Cell[66814, 1997, 284, 7, 54, "Input"],
Cell[67101, 2006, 149, 4, 32, "Input"],
Cell[67253, 2012, 5407, 144, 663, "Input"],
Cell[72663, 2158, 666, 18, 75, "Input"],
Cell[73332, 2178, 246, 6, 32, "Input"],
Cell[73581, 2186, 240, 5, 32, "Input"],
Cell[73824, 2193, 1574, 38, 285, "Input"],
Cell[75401, 2233, 249, 5, 32, "Input"],
Cell[75653, 2240, 2219, 57, 285, "Input"],
Cell[77875, 2299, 1495, 40, 222, "Input"],
Cell[79373, 2341, 1359, 35, 138, "Input"],
Cell[80735, 2378, 234, 6, 32, "Input"],
Cell[80972, 2386, 343, 9, 54, "Input"],
Cell[81318, 2397, 2694, 71, 264, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[84049, 2473, 49, 0, 50, "Section"],
Cell[84101, 2475, 968, 27, 117, "Input"],
Cell[85072, 2504, 178, 4, 32, "Input"],
Cell[85253, 2510, 1714, 46, 222, "Input"],
Cell[86970, 2558, 259, 7, 32, "Input"],
Cell[87232, 2567, 12107, 304, 1083, "Input"],
Cell[99342, 2873, 6928, 165, 894, "Input"],
Cell[106273, 3040, 201, 4, 32, "Input"],
Cell[106477, 3046, 1381, 36, 222, "Input"]
}, Closed]]
}
]
*)

